package de.szut.casino.blackjack;

import de.szut.casino.user.UserEntity;
import de.szut.casino.user.UserRepository;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Random;

@Service
public class BlackJackService {
    private final BlackJackGameRepository blackJackGameRepository;
    private final UserRepository userRepository;

    public BlackJackService(BlackJackGameRepository blackJackGameRepository, UserRepository userRepository) {
        this.blackJackGameRepository = blackJackGameRepository;
        this.userRepository = userRepository;
    }

    private final Random random = new Random();

    public BlackJackGameEntity createBlackJackGame(UserEntity user, BigDecimal betAmount) {
        BlackJackGameEntity game = new BlackJackGameEntity();
        game.setUser(user);
        game.setBet(betAmount);
        game.setState(getState(game));
        initializeDeck(game);

        for (int i = 0; i < 2; i++) {
            CardEntity playerCard = drawCardFromDeck(game);
            game.getPlayerCards().add(playerCard);
        }

        CardEntity dealerCard = drawCardFromDeck(game);
        game.getDealerCards().add(dealerCard);

        user.setBalance(user.getBalance().subtract(betAmount));

        userRepository.save(user);
        blackJackGameRepository.save(game);

        return game;
    }

    private void initializeDeck(BlackJackGameEntity game) {
        for (Suit suit : Suit.values()) {
            for (Rank rank : Rank.values()) {
                CardEntity card = new CardEntity();
                card.setGame(game);
                card.setSuit(suit);
                card.setRank(rank);
                game.getDeck().add(card);
            }
        }

        java.util.Collections.shuffle(game.getDeck(), random);
    }

    private CardEntity drawCardFromDeck(BlackJackGameEntity game) {
        if (game.getDeck().isEmpty()) {
            throw new IllegalStateException("Deck is empty");
        }

        return game.getDeck().removeFirst();
    }

    private BlackJackState getState(BlackJackGameEntity game) {
        return BlackJackState.IN_PROGRESS;
    }
}
